--
-- GuildArtisanRow
--
-- Copyright (c) 2021 Dana Basken
--

local AceEvent = LibStub("AceEvent-3.0")

GuildArtisanRow = {}
GuildArtisanRow.__index = GuildArtisanRow

function GuildArtisanRow:Create(name, icon, frame, data, renderer, updater)
  local instance = {}
  setmetatable(instance, GuildArtisanRow)
  instance:Construct(name, icon, frame, data, renderer, updater)
  return instance
end

function GuildArtisanRow:Construct(name, icon, frame, data, renderer, updater)
  self.name = name
  self.icon = icon
  self.parentFrame = frame
  self.data = data
  self.renderer = renderer
  self.updater = updater
  if name then self.expanded = false else self.expanded = true end
  self.children = {}
  self:CreateUI()
  self:UpdateUI()
end

function GuildArtisanRow:CreateUI()
  self.frame = CreateFrame("Frame", nil, self.parentFrame, BackdropTemplateMixin and "BackdropTemplate")
  self.frame:SetWidth(self.parentFrame:GetWidth())
  self.frame:SetHeight(0)
  self.frame:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", tile = true, tileSize = 32})
  if self.name then
    self.frame:SetBackdropColor(0.25, 0, 0.25, 0)
  else
    self.frame:SetBackdropColor(0, 0, 0, 0.25)
  end
  self:SetPosition(0, 0)
  self.frame:SetScript("OnMouseDown", function (_, button)
    if button == 'LeftButton' then self:ToggleExpanded() end
  end)
  if self.name then self:CreateDataUI() end
  self:CreateChildUI()
end

function GuildArtisanRow:ToggleExpanded()
  if self.name then
    if self.expanded then self:Contract() else self:Expand() end
  end
end

function GuildArtisanRow:CreateDataUI()
  local rowHeight = 32
  self.dataFrame = CreateFrame("Frame", nil, self.frame, BackdropTemplateMixin and "BackdropTemplate")
  self.dataFrame:SetWidth(self.frame:GetWidth())
  self.dataFrame:SetHeight(rowHeight)
  self.dataFrame:SetPoint("TOPLEFT", self.frame, 0, 0)
  self.dataFrame:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", tile = true, tileSize = 32, insets = {left = 1, right = 1, top = 1, bottom = 1}})
  self.dataFrame:SetBackdropColor(0.25, 0, 0.25, 0)

  local position = 0
  local padding = 4
  if self.icon then
    position = position + padding
    local iconSize = rowHeight * 0.6
    self.dataFrame.icon = CreateFrame("Frame", nil, self.dataFrame)
    self.dataFrame.icon:SetWidth(iconSize)
    self.dataFrame.icon:SetHeight(iconSize)
    self.dataFrame.icon:SetPoint("LEFT", self.dataFrame, position, 0)
    self.dataFrame.icon.texture = self.dataFrame.icon:CreateTexture()
    self.dataFrame.icon.texture:SetAllPoints()
    self.dataFrame.icon.texture:SetTexture(self.icon)
    position = position + iconSize
  end

  position = position + padding
  self.dataFrame.name = self.dataFrame:CreateFontString("FontString", "OVERLAY")
  self.dataFrame.name:SetFont("Fonts\\FRIZQT__.TTF", 10, "MONOCHROME")
  self.dataFrame.name:SetPoint("LEFT", self.dataFrame, position, 0)
  self.dataFrame.name:SetText(self.name)
  position = position + self.dataFrame.name:GetStringWidth()

  if self.renderer then
    position = position + padding
    self.dataFrame.data = CreateFrame("Frame", nil, self.dataFrame)
    self.dataFrame.data:SetHeight(rowHeight)
    self.dataFrame.data:SetPoint("LEFT", self.dataFrame, position, 0)
    self.renderer(nil, self.dataFrame.data, self.data)
    position = position + self.dataFrame.data:GetWidth()
  end
end

function GuildArtisanRow:CreateChildUI()
  self.childrenFrame = CreateFrame("Frame", nil, self.frame, BackdropTemplateMixin and "BackdropTemplate")
  self.childrenFrame:SetWidth(self.frame:GetWidth())
  self.childrenFrame:SetHeight(0)
  self.childrenFrame:SetPoint("TOPLEFT", self.frame, 0, 0)
end

function GuildArtisanRow:UpdateUI()
  -- print(self.name, ObjectToString({self:GetPosition()}), self:GetHeight())
  local height = 0
  if self.dataFrame then
    self:UpdateDataUI()
    height = height + self.dataFrame:GetHeight()
  end
  local childFrameHeight = self:UpdateChildUI(height)
  height = height + childFrameHeight
  self:SetHeight(height)
end

function GuildArtisanRow:UpdateDataUI()
  if self.updater then
    self.updater(nil, self.dataFrame.data, self.data)
  end
end

function GuildArtisanRow:UpdateData(data)
  self.data = data
  self:UpdateDataUI()
end

function GuildArtisanRow:UpdateChildUI(height)
  local childFrameHeight = 0
  for i = 1, #self.children do
    local child = self.children[i]
    child:UpdateUI()
    if self.expanded then
      child:SetPosition(10, height + childFrameHeight)
      child:Show()
      childFrameHeight = childFrameHeight + child:GetHeight()
    else
      child:SetPosition(0, 0)
      child:Hide()
    end
  end
  self.childrenFrame:SetHeight(childFrameHeight)
  return childFrameHeight
end

function GuildArtisanRow:AddChild(row)
  table.insert(self.children, row)
  row.parent = self
end

function GuildArtisanRow:RemoveChild(row)
  -- TODO
end

function GuildArtisanRow:GetHeight()
  return self.frame:GetHeight()
end

function GuildArtisanRow:SetHeight(height)
  self.frame:SetHeight(height)
end

function GuildArtisanRow:GetPosition()
  local point, relativeTo, relativePoint, xOfs, yOfs = self.frame:GetPoint()
  return {x = xOfs, y = yOfs, relative = relativePoint}
end

function GuildArtisanRow:SetPosition(x, y)
  self.frame:SetPoint("TOPLEFT", self.parentFrame, x, -y)
end

function GuildArtisanRow:UpdateParentUI()
  if self.parent then
    self.parent:UpdateUI()
    self.parent:UpdateParentUI()
  end
end

function GuildArtisanRow:Expand()
  if #self.children > 0 then
    self.expanded = true
    self:UpdateParentUI()
  end
end

function GuildArtisanRow:Contract()
  self.expanded = false
  self:UpdateParentUI()
end

function GuildArtisanRow:Show()
  self.frame:Show()
  self.childrenFrame:Show()
end

function GuildArtisanRow:Hide()
  self.frame:Hide()
end

function GuildArtisanRow:GetChildNamed(name)
  for index, row in pairs(self.children) do
    if row.name == name then return row end
  end
  return nil
end

function GuildArtisanRow:GetChildNames()
  local results = {}
  for index, row in pairs(self.children) do
    results[row.name] = true
  end
  return results
end
